package dev.fitko.fitconnect.api.domain.sender;

import dev.fitko.fitconnect.api.domain.model.attachment.Attachment;
import dev.fitko.fitconnect.api.domain.model.callback.Callback;
import dev.fitko.fitconnect.api.domain.model.metadata.AuthenticationInformation;
import dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType;
import dev.fitko.fitconnect.api.domain.model.metadata.data.SubmissionSchema;
import dev.fitko.fitconnect.api.domain.model.metadata.payment.PaymentInformation;
import dev.fitko.fitconnect.api.domain.model.reply.replychannel.ReplyChannel;
import dev.fitko.fitconnect.api.domain.sender.steps.reply.BuildReplyStep;
import dev.fitko.fitconnect.api.domain.sender.steps.reply.OptionalReplyPropertiesStep;
import dev.fitko.fitconnect.api.domain.sender.steps.reply.ReplyDataStep;
import dev.fitko.fitconnect.api.domain.subscriber.ReceivedSubmission;
import lombok.Getter;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

@Getter
public final class SendableReply {

    private final UUID destinationId;
    private final UUID caseId;
    private final UUID originalSubmissionId;
    private final String data;
    private final SubmissionSchema submissionSchema;
    private final List<Attachment> attachments;
    private final String serviceName;
    private final String serviceIdentifier;
    private final List<AuthenticationInformation> authenticationInformation;
    private final PaymentInformation paymentInformation;
    private final ReplyChannel replyChannel;
    private final Callback callback;

    private SendableReply(final SendableReply.Builder builder) {
        destinationId = builder.getDestinationId();
        caseId = builder.getCaseId();
        originalSubmissionId = builder.getOriginalSubmissionId();
        data = builder.getData();
        submissionSchema = new SubmissionSchema(builder.getSchemaUri(), builder.getDataMimeType());
        attachments = Collections.unmodifiableList(builder.getAttachments());
        serviceName = builder.getServiceName();
        serviceIdentifier = builder.getServiceIdentifier();
        authenticationInformation = builder.getAuthenticationInformation();
        replyChannel = builder.getReplyChannel();
        paymentInformation = builder.getPaymentInformation();
        callback = builder.getCallback();
    }

    public static ReplyDataStep from(final ReceivedSubmission receivedSubmission) {
        return new SendableReply.Builder(receivedSubmission);
    }

    @Getter
    private static final class Builder implements ReplyDataStep, OptionalReplyPropertiesStep, BuildReplyStep {

        private final UUID destinationId;
        private final UUID caseId;
        private final UUID originalSubmissionId;
        private final List<Attachment> attachments = new ArrayList<>();
        private String data;
        private MimeType dataMimeType;
        private final String serviceName;
        private final String serviceIdentifier;
        private final List<AuthenticationInformation> authenticationInformation;
        private final PaymentInformation paymentInformation;
        private final ReplyChannel replyChannel;
        private URI schemaUri;
        private final Callback callback;

        private Builder(final ReceivedSubmission receivedSubmission) {
            caseId = receivedSubmission.getCaseId();
            originalSubmissionId = receivedSubmission.getSubmissionId();
            destinationId = receivedSubmission.getDestinationId();
            serviceIdentifier = receivedSubmission.getServiceType().getIdentifier();
            serviceName = receivedSubmission.getServiceType().getName();
            callback = receivedSubmission.getCallback();
            replyChannel = receivedSubmission.getMetadata().getReplyChannel();
            authenticationInformation = receivedSubmission.getMetadata().getAuthenticationInformation();
            paymentInformation = receivedSubmission.getMetadata().getPaymentInformation();
        }

        public OptionalReplyPropertiesStep setJsonData(final String replyData, final URI replyDataSchemaUri) {
            data = replyData;
            schemaUri = replyDataSchemaUri;
            dataMimeType = MimeType.APPLICATION_JSON;
            return this;
        }

        public OptionalReplyPropertiesStep setXmlData(final String replyData, final URI replyDataSchemaUri) {
            data = replyData;
            schemaUri = replyDataSchemaUri;
            dataMimeType = MimeType.APPLICATION_XML;
            return this;
        }

        @Override
        public OptionalReplyPropertiesStep addAttachments(final List<Attachment> attachments) {
            attachments.addAll(attachments);
            return this;
        }

        @Override
        public OptionalReplyPropertiesStep addAttachment(final Attachment attachment) {
            attachments.add(attachment);
            return this;
        }

        @Override
        public SendableReply build() {
            return new SendableReply(this);
        }
    }
}
