package dev.fitko.fitconnect.api.domain.sender.steps.unencrypted;

import dev.fitko.fitconnect.api.domain.model.callback.Callback;
import dev.fitko.fitconnect.api.domain.model.metadata.AuthenticationInformation;
import dev.fitko.fitconnect.api.domain.model.metadata.payment.PaymentInformation;
import dev.fitko.fitconnect.api.domain.model.reply.replychannel.ReplyChannel;
import dev.fitko.fitconnect.api.domain.model.attachment.Attachment;
import dev.fitko.fitconnect.api.domain.sender.SendableSubmission;

import java.net.URI;
import java.util.List;
import java.util.UUID;

public interface OptionalPropertiesStep {

    /**
     * Sends the submission with a list of attachments
     *
     * @param attachments that are sent with the submission
     * @return the next step where additional data can be added to the submission
     */
    OptionalPropertiesStep addAttachments(List<Attachment> attachments);

    /**
     * Sends the submission with an attachments
     *
     * @param attachment that is sent with the submission
     * @return the next step where additional data can be added to the submission
     */
    OptionalPropertiesStep addAttachment(Attachment attachment);

    /**
     * Add a reply channel. The {@link ReplyChannel} object provides static factories for all available options:
     * <ul>
     * <li>ReplyChannel.fromEmail(...)</li>
     * <li>ReplyChannel.fromDeMail(...)</li>
     * <li>ReplyChannel.fromFink(...)</li>
     * <li>ReplyChannel.fromElster(...)</li>
     * <li>ReplyChannel.fromFitConnect(...)</li>
     * </ul>
     *
     * @param replyChannel a configured reply channel
     * @return next step to set more additional properties
     * @see <a href="https://docs.fitko.de/fit-connect/docs/metadata/replyChannel#r%C3%BCckkanal-replychannel">FIT-Connect documentation on reply channels</a>
     */
    OptionalPropertiesStep setReplyChannel(ReplyChannel replyChannel);

    /**
     * Set {@link AuthenticationInformation} that are necessary for the submission.
     *
     * @param authenticationInformation list of {@link AuthenticationInformation}
     * @return next step to set more additional properties
     * @see <a href="https://docs.fitko.de/fit-connect/docs/metadata/authenticationInformation">FIT-Connect documentation on authentication information</a>
     */
    OptionalPropertiesStep setAuthenticationInformation(List<AuthenticationInformation> authenticationInformation);

    /**
     * Set {@link PaymentInformation} that contains information on the payment service.
     * If the optional field is not set the service was not billed yet or is free of charge.
     *
     * @param paymentInformation the payment object containing transaction details
     * @return next step to set more additional properties
     * @see <a href="https://docs.fitko.de/fit-connect/docs/metadata/paymentInformation">FIT-Connect documentation on payment information</a>
     */
    OptionalPropertiesStep setPaymentInformation(PaymentInformation paymentInformation);

    /**
     * Set {@link Callback} to be informed about submission status updates asynchronous without polling.
     *
     * @param callbackUri    URI that should be called to notify the sender
     * @param callbackSecret Secret to verify the callback
     * @return next step to set more additional properties
     * @see <a href="https://docs.fitko.de/fit-connect/docs/details/callbacks#callback-url">FIT-Connect documentation on callbacks</a>
     */
    OptionalPropertiesStep setCallback(URI callbackUri, String callbackSecret);

    /**
     * Add a new submission to an existing case.
     *
     * @param caseId unique identifier of the submissions case
     * @return next step to set more additional properties
     */
    OptionalPropertiesStep setCase(UUID caseId);

    /**
     * Constructs a new sendable submission from the data that is set.
     *
     * @return SendableSubmission
     */
    SendableSubmission build();
}
