package dev.fitko.fitconnect.api.domain.subscriber;

import dev.fitko.fitconnect.api.FitConnectService;
import dev.fitko.fitconnect.api.domain.model.attachment.Attachment;
import dev.fitko.fitconnect.api.domain.model.callback.Callback;
import dev.fitko.fitconnect.api.domain.model.event.Event;
import dev.fitko.fitconnect.api.domain.model.event.EventPayload;
import dev.fitko.fitconnect.api.domain.model.event.SubmissionStatus;
import dev.fitko.fitconnect.api.domain.model.event.problems.Problem;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.submission.ServiceType;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.exceptions.client.FitConnectSubscriberException;
import lombok.AllArgsConstructor;
import lombok.Builder;

import java.net.URI;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static dev.fitko.fitconnect.api.domain.model.event.EventPayload.forAcceptEvent;
import static dev.fitko.fitconnect.api.domain.model.event.EventPayload.forAcceptEventWithAttachments;
import static dev.fitko.fitconnect.api.domain.model.event.EventPayload.forRejectEvent;

@Builder
@AllArgsConstructor
public class ReceivedSubmission implements Comparable<ReceivedSubmission> {

    private final transient FitConnectService subscriber;
    private final Submission submission;
    private final Metadata metadata;
    private final ReceivedData receivedData;
    private final List<Attachment> attachments;
    private final Map<UUID, String> encryptedAttachments;
    private final SubmissionStatus submitState;

    /**
     * Send an accept-submission {@link Event} to confirm that the handed in submission was accepted.
     *
     * @param problems optional {@link Problem}s
     * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">FIT-Connect Events</a>
     */
    public void acceptSubmission(final Problem... problems) {
        try {
            subscriber.acceptSubmission(getEventPayload(problems));
        } catch (final Exception e) {
            throw new FitConnectSubscriberException("Accepting submission failed", e.getCause() != null ? e.getCause() : e);
        }
    }

    /**
     * Send a reject-submission {@link Event} to confirm that the handed in submission was rejected.
     *
     * @param rejectionProblems list of {@link Problem}s that give more detailed information on why the submission was rejected
     * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">FIT-Connect Events</a>
     */
    public void rejectSubmission(final List<Problem> rejectionProblems) {
        try {
            subscriber.rejectSubmission(forRejectEvent(submission, rejectionProblems));
        } catch (final Exception e) {
            throw new FitConnectSubscriberException("Rejecting submission failed", e.getCause() != null ? e.getCause() : e);
        }
    }

    /**
     * Access decrypted data.
     *
     * @return data as string
     */
    public String getDataAsString() {
        return receivedData.getData();
    }

    /**
     * Gets data mime-type.
     *
     * @return mimetype string
     */
    public String getDataMimeType() {
        return receivedData.getMimeType().value();
    }

    /**
     * Gets the data schema reference.
     *
     * @return data schema as URI
     */
    public URI getDataSchemaUri() {
        return metadata.getContentStructure().getData().getSubmissionSchema().getSchemaUri();
    }

    /**
     * Gets the submission service type.
     *
     * @return {@link ServiceType} object
     */
    public ServiceType getServiceType() {
        return submission.getServiceType();
    }

    /**
     * Access list of decrypted attachments.
     *
     * @return list of {@link Attachment}
     */
    public List<Attachment> getAttachments() {
        return attachments;
    }

    /**
     * Gets the submission {@link Metadata}.
     *
     * @return metadata
     */
    public Metadata getMetadata() {
        return metadata;
    }

    /**
     * Gets the submission caseId.
     *
     * @return caseId as UUID
     */
    public UUID getCaseId() {
        return submission.getCaseId();
    }

    /**
     * Gets the submission destinationId.
     *
     * @return destinationId as UUID
     */
    public UUID getDestinationId() {
        return submission.getDestinationId();
    }

    /**
     * Gets the submission submissionId.
     *
     * @return submissionId as UUID
     */
    public UUID getSubmissionId() {
        return submission.getSubmissionId();
    }

    /**
     * Get the submissions' callback.
     *
     * @return callback
     */
    public Callback getCallback() {
        return submission.getCallback();
    }

    /**
     * Get the timestamp when the submission was submitted.
     *
     * @return timestamp as {@link Date}
     */
    public Date getSubmittedAt() {
        return submitState.getIssuedAt();
    }

    private EventPayload getEventPayload(final Problem[] problems) {
        return attachments.isEmpty() ? forAcceptEvent(submission, problems) : forAcceptEventWithAttachments(submission, encryptedAttachments, problems);
    }

    @Override
    public int compareTo(final ReceivedSubmission other) {
        return getSubmittedAt().compareTo(other.getSubmittedAt());
    }
}
